package cn.ecpark.service.usergw.biz.filters.factory;

import java.nio.ByteBuffer;
import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import com.alibaba.fastjson.JSON;

import org.apache.dubbo.rpc.service.GenericService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilter;
import org.springframework.cloud.gateway.filter.factory.AbstractGatewayFilterFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.http.HttpHeaders;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.stereotype.Component;

import cn.ecpark.service.usergw.biz.filters.bean.GenericServicePool;
import lombok.extern.slf4j.Slf4j;
import reactor.core.publisher.Mono;;

@Component
@Slf4j
public class DubboGatewayFilterFactory extends AbstractGatewayFilterFactory<DubboGatewayFilterFactory.Config> {

    @Autowired
    private ApplicationContext appContext;

    /**
     * DUBBO_URI key.
     */
    public static final String DUBBO_URI = "dubbo_uri";
    public static HashMap<String, List<String>> mehtods = new HashMap<>();

    private List<String> emptyList = new LinkedList<String>();

    public DubboGatewayFilterFactory() {
        super(Config.class);
    }

    @Override
    public List<String> shortcutFieldOrder() {
        return Arrays.asList(DUBBO_URI);
    }

    @Override
    public GatewayFilter apply(Config config) {
        String uri = config.dubboUri;
        return (exchange, chain) -> {

            ServerHttpRequest req = exchange.getRequest();
            HttpHeaders headers = req.getHeaders();

            List<String> methodString = headers.get("method");
            List<String> params = headers.get("params");

            Object result = null;
            ServerHttpResponse response = exchange.getResponse();

            if (methodString.size() != 0) {
                List<String> paramTypes;
                // 判断全部函数允许, 必须带参数类型, 而且要匹配, 否则报错
                if (uri.charAt(5) == '-') {
                    paramTypes = headers.get("param-types");
                    if(paramTypes == null) {
                        paramTypes = this.emptyList;
                    }
                } else {
                    paramTypes = mehtods.get(methodString.get(0));
                }

                if (paramTypes != null) {
                    int paramsSize = 0;
                    if (params != null) {
                        paramsSize = params.size();
                    }

                    if (paramTypes.size() == paramsSize) {

                        GenericServicePool gsPool = appContext.getBean(GenericServicePool.class);
                        GenericService gs = gsPool.get(uri);

                        if (paramsSize == 0) {
                            result = gs.$invoke(methodString.get(0), new String[] {}, new Object[] {});
                        } else {
                            result = gs.$invoke(methodString.get(0),
                                    Arrays.copyOf(paramTypes.toArray(), paramTypes.size(), String[].class),
                                    params.toArray());
                        }
                        if (result == null) {
                            return response.setComplete();
                        }
                    } else {
                        result = String.format("paramTypes.size %d is not equals to params size %d", paramTypes.size(),
                                paramsSize);
                        log.warn((String) result);
                    }
                } else {
                    result = String.format("(mehtod: %s, param-types: null)  is not exist or method is not allowed",
                            methodString.get(0));
                    log.warn((String) result);
                }
            } else {
                result = String.format("queryParams.get(\"method\") is null");
                log.warn((String) result);
            }

            if (result != null) {
                if (result.getClass() == String.class) {
                    return response.writeWith(
                            Mono.just(response.bufferFactory().wrap(ByteBuffer.wrap(((String) result).getBytes()))));
                } else {
                    return response.writeWith(Mono.just(
                            response.bufferFactory().wrap(ByteBuffer.wrap(JSON.toJSONString(result).getBytes()))));
                }
            }

            return chain.filter(exchange);
        };
    }

    public static class Config {

        private String dubboUri;

        public String getDubboUri() {
            return dubboUri;
        }

        public void setDubboUri(String dubboUri) {
            this.dubboUri = dubboUri;
        }

    }

}