package cn.ecpark.service.usergw.config;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import org.apache.dubbo.config.ApplicationConfig;
import org.apache.dubbo.config.ReferenceConfig;
import org.apache.dubbo.config.RegistryConfig;
import org.apache.dubbo.rpc.service.GenericService;

import cn.ecpark.service.usergw.biz.filters.factory.DubboGatewayFilterFactory;
import cn.ecpark.service.usergw.utils.Convert;
import lombok.extern.slf4j.Slf4j;

/**
 * ConfigSpecialFunction
 */
@Slf4j
public class ConfigSpecialFunction {

    /**
     * 默认调用ref的函数, 通过反射方法
     * @param ref {@link ReferenceConfig} {@link GenericService} 泛调用的接口
     * @param methodName 方法名 用于调用接口的方法
     * @param params 方法的参数
     * @throws IllegalAccessException 
     * @throws IllegalArgumentException 
     * @throws InvocationTargetException
     * @throws Exception
     */
    public static void setDefault(ReferenceConfig<GenericService> ref, String methodName, Object params)
            throws Exception {
        methodName = "set" + Convert.firstUpperCase(methodName);
        log.info("method:{}, value:{}", methodName, params);
        Method method = ref.getClass().getMethod(methodName, params.getClass());
        method.invoke(ref, params);
    }

    /**
     * 遇到application 做特殊的 函数处理
     * @param ref {@link ReferenceConfig} {@link GenericService} 泛调用的接口
     * @param params 传递参数 {@link ReferenceConfig#setApplication(ApplicationConfig)}
     */
    public static void setApplication(ReferenceConfig<GenericService> ref, Object params) {
        ref.setApplication(new ApplicationConfig((String) params));
    }

    /**
     * 遇到registry 做特殊的 函数处理
     * @param ref {@link ReferenceConfig} {@link GenericService} 泛调用的接口
     * @param params 传递参数 {@link ReferenceConfig#setRegistry(RegistryConfig)}
     */
    public static void setRegistry(ReferenceConfig<GenericService> ref, Object params) {
        ref.setRegistry(new RegistryConfig((String) params));
    }

    /**
     * 把设置method域的一些 函数名 参数类型 缓存起来. 没设置的运行通过
     * @param ref {@link ReferenceConfig} {@link GenericService} 泛调用的接口
     * @param params 传递参数 {@link ReferenceConfig#setApplication(ApplicationConfig)}
     */
    @SuppressWarnings("unchecked")
    public static void setMethods(ReferenceConfig<GenericService> ref, Object params) {
        List<HashMap<String, Object>> cfgs = (List<HashMap<String, Object>>) params;
        for (HashMap<String, Object> cfg : cfgs) {
            String name = (String) cfg.get("name");
            if (DubboGatewayFilterFactory.mehtods.containsKey(name)) {
                log.warn("method {} is duplicate", name);
            }
            List<String> paramTypes = (List<String>) cfg.get("param-types");
            if (paramTypes == null) {
                paramTypes = new LinkedList<>(); // 不用ArrayList 减少内存的使用.
            }
            DubboGatewayFilterFactory.mehtods.put(name, paramTypes);
        }

    }

}